%
%    Copyright (C) 2008  Colin Caprani - www.colincaprani.com
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

function SP = secprop(outer,inner,plotflag)
% Section Properties Calculator
% outer is a matrix of outer coordinates (x,y)
% inner is a matrix of coordinates for a void
% SP is a vector of A,I,yt and yb properties of the section

    [nOC n] = size(outer); [nIC n] = size(inner);
    outer(nOC+1,:) = outer(1,:);
    inner(nIC+1,:) = inner(1,:);
   
    propsOC = zeros(1,3); propsIC = propsOC;
    propsOC = algor(outer);
    if(nIC > 2); propsIC = algor(inner); end;
    A = propsOC(1) - propsIC(1);
    I = propsOC(2) - propsIC(2);
    ybar = propsOC(3) - propsIC(3);
    
    ybar = ybar/A;
    I = I-A*ybar^2;   
    A = 2*A; I = 2*I;
    yt = max(outer(:,2)) - ybar;
    yb = ybar;
    SP = [A,I,yt,yb]; SP = SP';
    
    if plotflag == 1
        outer(nOC+1:2*nOC-1,1) = -outer(nOC:-1:2,1);
        outer(nOC+1:2*nOC-1,2) =  outer(nOC:-1:2,2);
        inner(nIC+1:2*nIC-1,1) = -inner(nIC:-1:2,1);
        inner(nIC+1:2*nIC-1,2) =  inner(nIC:-1:2,2);
        h = fill(outer(:,1),outer(:,2),'r'); hold on;
        fill(inner(:,1),inner(:,2),'w'); axis equal;

        s1 = sprintf('Area = %d mm2 | Height to N-A = %3.1f mm',A,round(ybar*10)/10);
        s2 = sprintf('Second Moment of Area = %d mm4',I);
        s3 = sprintf('Elastic Moduli, t = %d mm3; b = %d mm3',I/yt,I/yb);
        annotation1 = annotation('textbox',[0.15 0.12 0.4 0.14],...
          'FitHeightToText','on','EdgeColor','none','String',{s1,s2,s3});
    end
end

function props = algor(vc)
    A = 0; ybar = 0; I = 0;
    x = vc(:,1); y = vc(:,2);
    n = length(x);
    for i = 1:(n-1)
        A = A + 0.5*(x(i)-x(i+1))*(y(i)+y(i+1));
        ybar = ybar + (1/6)*(x(i)-x(i+1))*(y(i)^2 + y(i)*y(i+1) + y(i+1)^2);
        I = I + (1/12)*(x(i)-x(i+1))*(y(i)^3 + y(i)^2*y(i+1)+y(i)*y(i+1)^2 + y(i+1)^3);
    end
    props = [A,I,ybar];
end
    